with Ada.Text_IO; use Ada.Text_IO;
with Ada.Calendar; use Ada.Calendar;
with Ada.Numerics.Float_Random, Version1, Version2, Version3, Version4, Version5, Version6;
use Ada.Numerics.Float_Random, Version1, Version2, Version3, Version4, Version5, Version6;
-- Test/Compare 7 random number generators
procedure Test_Rand is
   No_Calls: constant := 10000000;
   First_Few: constant := 10;

   Gen0: Ada.Numerics.Float_Random.Generator; -- version on GNAT (in this case)
   Gen1: Version1.Generator; -- ALGORITHM AS 183 APPL. STATIST. (1982) VOL.31, NO.2, 32-bit version
   Gen2: Version2.Generator; -- ALGORITHM AS 183 APPL. STATIST. (1982) VOL.31, NO.2, 16-bit version
   Gen3, Gen3a: Version3.Generator; -- New 3-cycle generator, 64-bit version
   Gen4, Gen4a: Version4.Generator; -- New 3-cycle generator, 32-bit version
   Gen5, Gen5a: Version5.Generator; -- New 4-cycle generator, 64-bit version
   Gen6, Gen6a: Version6.Generator; -- New 4-cycle generator, 32-bit version

   -- Versions 1 and 2 are equivalent, as are Versions 3 and 4, and Versions 5 and 6.

   State0: Ada.Numerics.Float_Random.State;

   Start: Ada.Calendar.Time;
   Count_Half: Integer;

package My_Int is new Integer_IO(Integer);
use My_Int;
package My_Flt is new Float_IO(Long_Float);
use My_Flt;

Value: Long_Float;

procedure Finish(Ver: Integer) is
   Interval: Long_Float := Long_Float(Duration'(Ada.Calendar.Clock - Start));
   begin
   Put("Number less than half: ");
   Put(Count_Half, Width => 1);
   Put(" from ");
   Put(No_Calls, Width => 1);
   New_Line;
   Put("Speed of Generator ");
   Put(Ver, Width => 1);
   New_Line;
   Put("Number of calls per second ");
   Put( Integer(Long_Float(No_calls)/Interval), Width => 1);
   New_Line;
   New_Line;
   end Finish;

begin
Put("Gen0"); -- Version is for Float, rather than Long_Float
New_Line;
Ada.Numerics.Float_Random.Reset(Gen0, 12345);
Ada.Numerics.Float_Random.Save(Gen0, State0);
if Ada.Numerics.Float_Random.Value(Ada.Numerics.Float_Random.Image(State0)) /= State0 then
   Put("Image failure for Gen0");
   New_Line;
end if;
for I in Integer range 1..First_Few loop
   Value := Long_Float(Ada.Numerics.Float_Random.Random(Gen0));
   Put(Value, Exp=>0);
   New_Line; 
end loop;
Count_Half := 0;
Start := Ada.Calendar.Clock;
for I in Integer range 1..No_Calls loop
   Value := Long_Float(Ada.Numerics.Float_Random.Random(Gen0));
   if Value < 0.5 then
      Count_Half := Count_Half + 1;
   end if;
end loop;
Finish(0);
 
Put("Gen1"); -- Version 1 testing, others are the same.
New_Line;
Version1.Reset(Gen1, 12345);
if Version1.Value(Version1.Image(Gen1)) /= Gen1 then
   Put("Image failure for Gen1");
   New_Line;
end if; 
for I in Integer range 1..First_Few loop
   Version1.Random(Gen1, Value);
   Put(Value, Exp=>0);
   New_Line; 
end loop; 
Count_Half := 0;
Start := Ada.Calendar.Clock;
for I in Integer range 1..No_Calls loop
   Version1.Random(Gen1, Value);
   if Value < 0.5 then
      Count_Half := Count_Half + 1;
   end if;
end loop;
Finish(1);
 
Put("Gen2");
New_Line;
Version2.Reset(Gen2, 12345);
if Version2.Value(Version2.Image(Gen2)) /= Gen2 then
   Put("Image failure for Gen2");
   New_Line;
end if; 
for I in Integer range 1..First_Few loop
   Version2.Random(Gen2, Value);
   Put(Value, Exp=>0);
   New_Line; 
end loop; 
Count_Half := 0;
Start := Ada.Calendar.Clock;
for I in Integer range 1..No_Calls loop
   Version2.Random(Gen2, Value);
   if Value < 0.5 then
      Count_Half := Count_Half + 1;
   end if;
end loop;
Finish(2);
 
Put("Gen3");
New_Line;
Version3.Reset(Gen3, 12345);
if Version3.Value(Version3.Image(Gen3)) /= Gen3 then
   Put("Image failure for Gen3");
   New_Line;
end if; 
for I in Integer range 1..First_Few loop
   Version3.Random(Gen3, Value);
   Put(Value, Exp=>0);
   Put(Version3.Image(Gen3));
   New_Line; 
end loop; 
Put("Print Reset for small integers");
New_Line;
for I in Integer range 1..First_Few loop
   Version3.Reset(Gen3, I);
   Version3.Random(Gen3, Value);
   Put(Value, Exp=>0);
   Put(Version3.Image(Gen3));
   New_Line; 
end loop; 
Put("Print values from new generator");
New_Line;
for I in Integer range 1..First_Few loop
   Gen3a := Version3.Next_Sequence(Gen3);
   Version3.Random(Gen3a, Value);
   Put(Value, Exp=>0);
   Put(Version3.Image(Gen3a));
   Gen3 := Gen3a;
   New_Line; 
end loop; 
Count_Half := 0;
Start := Ada.Calendar.Clock;
for I in Integer range 1..No_Calls loop
   Version3.Random(Gen3, Value);
   if Value < 0.5 then
      Count_Half := Count_Half + 1;
   end if;
end loop;
Finish(3);
 
Put("Gen4");
New_Line;
Version4.Reset(Gen4, 12345);
if Version4.Value(Version4.Image(Gen4)) /= Gen4 then
   Put("Image failure for Gen4");
   New_Line;
end if; 
for I in Integer range 1..First_Few loop
   Version4.Random(Gen4, Value);
   Put(Value, Exp=>0);
   Put(Version4.Image(Gen4));
   New_Line; 
end loop; 
Put("Print Reset for small integers");
New_Line;
for I in Integer range 1..First_Few loop
   Version4.Reset(Gen4, I);
   Version4.Random(Gen4, Value);
   Put(Value, Exp=>0);
   Put(Version4.Image(Gen4));
   New_Line; 
end loop; 
Put("Print values from new generator");
New_Line;
for I in Integer range 1..First_Few loop
   Gen4a := Version4.Next_Sequence(Gen4);
   Version4.Random(Gen4a, Value);
   Put(Value, Exp=>0);
   Put(Version4.Image(Gen4a));
   Gen4 := Gen4a;
   New_Line; 
end loop; 
Count_Half := 0;
Start := Ada.Calendar.Clock;
for I in Integer range 1..No_Calls loop
   Version4.Random(Gen4, Value);
   if Value < 0.5 then
      Count_Half := Count_Half + 1;
   end if;
end loop;
Finish(4);

Put("Gen5");
New_Line;
Version5.Reset(Gen5, 12345);
if Version5.Value(Version5.Image(Gen5)) /= Gen5 then
   Put("Image failure for Gen5");
   New_Line;
end if; 
for I in Integer range 1..First_Few loop
   Version5.Random(Gen5, Value);
   Put(Value, Exp=>0);
   Put(Version5.Image(Gen5));
   New_Line; 
end loop; 
Put("Print Reset for small integers");
New_Line;
for I in Integer range 1..First_Few loop
   Version5.Reset(Gen5, I);
   Version5.Random(Gen5, Value);
   Put(Value, Exp=>0);
   Put(Version5.Image(Gen5));
   New_Line; 
end loop; 
Put("Print 2 values from start");
Gen5 := Version5.Value("          1          1          1          1");
New_Line;
Version5.Random(Gen5, Value);
Put(Value, Exp=>0);
Put(Version5.Image(Gen5));
New_Line;
Version5.Random(Gen5, Value);
Put(Value, Exp=>0);
Put(Version5.Image(Gen5));
New_Line;
Put("Print values from new generator");
New_Line;
Gen5 := Version5.Value("          1          1          1          1");
for I in Integer range 1..First_Few loop
   Gen5a := Version5.Next_Sequence(Gen5);
   Version5.Random(Gen5a, Value);
   Put(Value, Exp=>0);
   Put(Version5.Image(Gen5a));
   Version5.Random(Gen5, Value);
   if Version5.Image(Gen5a) /= Version5.Image(Version5.Next_Sequence(Gen5)) then
      Put("Row and column sequences not commutative");
      Put(Version5.Image(Version5.Next_Sequence(Gen5)));
      New_Line;
   end if; 
   Gen5 := Gen5a;
   New_Line; 
end loop; 
Put("Print values from using the clock - should look random");
New_Line;
for I in Integer range 1..First_Few loop
   Version5.Reset(Gen5);
   Version5.Random(Gen5, Value);
   Put(Value, Exp=>0);
   Put(Version5.Image(Gen5));
   New_Line; 
end loop; 
Count_Half := 0;
Start := Ada.Calendar.Clock;
for I in Integer range 1..No_Calls loop
   Version5.Random(Gen5, Value);
   if Value < 0.5 then
      Count_Half := Count_Half + 1;
   end if;
end loop;
Finish(5);

Put("Gen6"); -- The recommended generator
New_Line;
Version6.Reset(Gen6, 12345);
if Version6.Value(Version6.Image(Gen6)) /= Gen6 then
   Put("Image failure for Gen6");
   New_Line;
end if; 
for I in Integer range 1..First_Few loop
   Version6.Random(Gen6, Value);
   Put(Value, Exp=>0);
   Put(Version6.Image(Gen6));
   New_Line; 
end loop; 
Put("Print Reset for small integers");
New_Line;
for I in Integer range 1..First_Few loop
   Version6.Reset(Gen6, I);
   Version6.Random(Gen6, Value);
   Put(Value, Exp=>0);
   Put(Version6.Image(Gen6));
   New_Line; 
end loop; 
Put("Print 2 values from start");
Gen6 := Version6.Value("          1          1          1          1");
New_Line;
Version6.Random(Gen6, Value);
Put(Value, Exp=>0);
Put(Version6.Image(Gen6));
New_Line;
Version6.Random(Gen6, Value);
Put(Value, Exp=>0);
Put(Version6.Image(Gen6));
New_Line;
Put("Print values from new generator");
Gen6 := Version6.Value("          1          1          1          1");
New_Line;
for I in Integer range 1..First_Few loop
   Gen6a := Version6.Next_Sequence(Gen6);
   Version6.Random(Gen6a, Value);
   Put(Value, Exp=>0);
   Put(Version6.Image(Gen6a));
   Version6.Random(Gen6, Value);
   if Version6.Image(Gen6a) /= Version6.Image(Version6.Next_Sequence(Gen6)) then
      Put("Row and column sequences not commutative");
      Put(Version6.Image(Version6.Next_Sequence(Gen6)));
      New_Line;
   end if; 
   Gen6 := Gen6a;
   New_Line; 
end loop; 
Put("Print values from using the clock - should look random");
New_Line;
for I in Integer range 1..First_Few loop
   Version6.Reset(Gen6);
   Version6.Random(Gen6, Value);
   Put(Value, Exp=>0);
   Put(Version6.Image(Gen6));
   New_Line; 
end loop; 
Count_Half := 0;
Start := Ada.Calendar.Clock;
for I in Integer range 1..No_Calls loop
   Version6.Random(Gen6, Value);
   if Value < 0.5 then
      Count_Half := Count_Half + 1;
   end if;
end loop;
Finish(6);
end Test_Rand;
 