with Ada.Calendar, Ada.Text_IO;
package body Version2 is

 procedure Random (Gen: in out Generator; Value: out Uniformly_Distributed) is
    -- ALGORITHM AS 183 APPL. STATIST. (1982) VOL.31, NO.2
    -- 16 bit version
    W: Long_Float;
    begin
    Gen.ix := 171 * (Gen.ix mod 177) - 2 * (Gen.ix/177);
    Gen.iy := 172 * (Gen.iy mod 176) - 35* (Gen.iy/176);
    Gen.iz := 170 * (Gen.iz mod 178) - 63* (Gen.iz/178);
    if Gen.ix < 0 then
       Gen.ix := Gen.ix + 30_269;
    end if;
    if Gen.iy < 0 then
       Gen.iy := Gen.iy + 30_307;
    end if;
    if Gen.iz < 0 then
       Gen.iz := Gen.iz + 30_323;
    end if;
    W := Long_Float(Gen.ix)/30_269.0 + Long_Float(Gen.iy)/30_307.0 + Long_Float(Gen.iz)/30_323.0;
    Value := W - Long_Float (Integer (W - 0.5));
    end Random;

 procedure Reset (Gen       : out Generator;
                  Initiator : in Integer) is
    Temp: Integer := Initiator/30_000;
    begin
    Gen.ix := Initiator mod 30_000 + 1;
    Gen.iy := Temp mod 30_000 + 1;
    Gen.iz := Temp/30_000 mod 30_000 + 1;
    end Reset;

 procedure Reset(Gen : out Generator) is
    -- use the Clock to reset the Generator
    T: Ada.Calendar.Time :=  Ada.Calendar.Clock;
    begin
    Gen.ix := Ada.Calendar.Month(T);
    Gen.iy := Ada.Calendar.Day(T);
    Gen.iz := Integer(Ada.Calendar.Seconds(T)/29 ) + 1; -- dividing to ensure result in range
    end Reset; 

 function Image (Gen   : in Generator) return String_State is
    Temp : constant String := Integer'Image(Gen.ix) & Integer'Image(Gen.iy) &
                              Integer'Image(Gen.iz);
    Temp2: String_State := (others => ' ');
    begin
    Temp2(1..Temp'Last) := Temp;
    return Temp2;
    end Image;
 
 function Value (Coded_State: String_State) return Generator is
    Gen: Generator;
    package My_IO is new Ada.Text_IO.Integer_IO(Integer);
    Last: Integer;
    begin
    My_IO.Get(Coded_State, Gen.ix, Last);
    if Gen.ix < 1 or Gen.ix >= 30_269 then
       raise Constraint_Error;
    end if;
    My_IO.Get(Coded_State(Last+1..Coded_State'Last), Gen.iy, Last);
    if Gen.iy < 1 or Gen.iy >= 30_307 then
       raise Constraint_Error;
    end if;
    My_IO.Get(Coded_State(Last+1..Coded_State'Last), Gen.iz, Last);
    if Gen.iz < 1 or Gen.iz >= 30_323 then
       raise Constraint_Error;
    end if;
    return Gen;
    end Value; 

end Version2;


