with Ada.Calendar, Ada.Text_IO;
package body Version4 is

 procedure Random (Gen: in out Generator; Value: out Uniformly_Distributed) is
    -- New Wichmann-Hill generator
    -- 32 bit version
    Div_2_147_483_579: constant := 1.0/2_147_483_579.0;
    Div_2_147_483_543: constant := 1.0/2_147_483_543.0;
    Div_2_147_483_423: constant := 1.0/2_147_483_423.0;
    W: Long_Float;
    begin
    Gen.ix := 46_341 * (Gen.ix mod 46_340) - 41_639* (Gen.ix/46_340);
    Gen.iy := 47_003 * (Gen.iy mod 45_688) - 10_479* (Gen.iy/45_688);
    Gen.iz := 45_762 * (Gen.iz mod 46_927) - 10_049* (Gen.iz/46_927);
    if Gen.ix < 0 then
       Gen.ix := Gen.ix + 2_147_483_579;
    end if;
    if Gen.iy < 0 then
       Gen.iy := Gen.iy + 2_147_483_543;
    end if;
    if Gen.iz < 0 then
       Gen.iz := Gen.iz + 2_147_483_423;
    end if;
    W := Long_Float(Gen.ix)*Div_2_147_483_579 + Long_Float(Gen.iy)*Div_2_147_483_543 + Long_Float(Gen.iz)*Div_2_147_483_423;
    Value := W - Long_Float (Integer (W - 0.5));
    end Random;

 procedure Reset (Gen       : out Generator;
                  Initiator : in Positive) is
    -- Equivalent to applying Next_Sequence Initiator times to
    -- a starting value using Initiator.
    function Exp(Mult, Power, Prime: Long_Long_Integer) return Long_Long_Integer is
       -- return Mult**Power mod Prime
       -- Uses the binary power method
       Temp: Long_Long_Integer := Power;
       MP: Long_Long_Integer := Mult;
       So_Far: Long_Long_Integer := 1;
       begin
       while Temp > 0 loop
          if Temp/2*2 /= Temp then
             So_Far := (So_Far*MP) mod Prime;
          end if;
          MP := (MP*MP) mod Prime;
          Temp := Temp/2;
       end loop;
       return So_Far;
       end Exp;
    begin
    Gen.ix := Integer(9_876_543*Exp(46_340, Long_Long_Integer(Initiator), 2_147_483_579) mod 2_147_483_579);
    Gen.iy := Integer(4_789_321*Exp(22_000, Long_Long_Integer(Initiator), 2_147_483_543) mod 2_147_483_543);
    Gen.iz := 1_345_678_901;
    end Reset;

 procedure Reset(Gen : out Generator) is
    -- use the Clock to reset the Generator
    T: Ada.Calendar.Time :=  Ada.Calendar.Clock;
    Value: Uniformly_Distributed;
    begin
    Gen.ix := Ada.Calendar.Month(T);
    Gen.iy := Ada.Calendar.Day(T);
    Gen.iz := Integer(Ada.Calendar.Seconds(T)*2400) + 1; -- multiple to cover range of result
    Random(Gen, Value); -- call to randomise value slightly better!
    Random(Gen, Value);                                                     
    end Reset; 

 function Image (Gen   : in Generator) return String_State is
    Temp : constant String := Integer'Image(Gen.ix) &
                              Integer'Image(Gen.iy) &
                              Integer'Image(Gen.iz);
    Temp2: String_State := (others => ' ');
    begin
    Temp2(1..Temp'Last) := Temp;
    return Temp2;
    end Image;
 
 function Value (Coded_State: String_State) return Generator is
    Gen: Generator;
    package My_IO is new Ada.Text_IO.Integer_IO(Integer);
    Last: Integer;
    begin
    My_IO.Get(Coded_State, Gen.ix, Last);
    if Gen.ix < 1 or Gen.ix >= 2_147_483_579 then
       raise Constraint_Error;
    end if;
    My_IO.Get(Coded_State(Last+1..Coded_State'Last), Gen.iy, Last);
    if Gen.iy < 1 or Gen.iy >= 2_147_483_543 then
       raise Constraint_Error;
    end if;
    My_IO.Get(Coded_State(Last+1..Coded_State'Last), Gen.iz, Last);
    if Gen.iz < 1 or Gen.iz >= 2_147_483_423 then
       raise Constraint_Error;
    end if;
    return Gen;
    end Value; 

 function Next_Sequence(Gen   : in Generator) return Generator is
    ix, iy: Integer;
    begin
    ix := 46_340 * (Gen.ix mod 42_948) - 40_631 * (Gen.ix / 42_948);
    iy := 22_000 * (Gen.iy mod 97_612) - 19_543 * (Gen.iy / 97_612);
    if ix < 0 then
       ix := ix + 2_147_483_579;
    end if;
    if iy < 0 then
       iy := iy + 2_147_483_543;
    end if;
    return (ix, iy, Gen.iz);
    end Next_Sequence;


end Version4;


