with Ada.Calendar, Ada.Text_IO;
package body Version5 is

 procedure Random (Gen: in out Generator; Value: out Uniformly_Distributed) is
    -- New Wichmann-Hill generator
    -- 64 bit version, 4 cycles
    Div_2_147_483_579: constant := 1.0/2_147_483_579.0;
    Div_2_147_483_543: constant := 1.0/2_147_483_543.0;
    Div_2_147_483_423: constant := 1.0/2_147_483_423.0;
    Div_2_147_483_123: constant := 1.0/2_147_483_123.0;
    W: Long_Float;
    begin
    Gen.ix := (11_600 * Gen.ix) mod 2_147_483_579;
    Gen.iy := (47_003 * Gen.iy) mod 2_147_483_543;
    Gen.iz := (23_000 * Gen.iz) mod 2_147_483_423;
    Gen.iu := (33_000 * Gen.iu) mod 2_147_483_123;
    W := Long_Float(Gen.ix)*Div_2_147_483_579 + Long_Float(Gen.iy)*Div_2_147_483_543
       + Long_Float(Gen.iz)*Div_2_147_483_423 + Long_Float(Gen.iu)*Div_2_147_483_123;
    -- Value := W - Long_Float (Integer (W - 0.5));
    while W >= 1.0 loop
       W := W - 1.0;
    end loop;
    Value := W;
    end Random;

 procedure Reset (Gen       : out Generator;
                  Initiator : in Positive) is
    -- Equivalent to applying Next_Sequence Initiator times to
    -- a starting value using Initiator.
    function Exp(Mult, Power, Prime: Long_Long_Integer) return Long_Long_Integer is
       -- return Mult**Power mod Prime
       -- Uses the binary power method
       Temp: Long_Long_Integer := Power;
       MP: Long_Long_Integer := Mult;
       So_Far: Long_Long_Integer := 1;
       begin
       while Temp > 0 loop
          if Temp/2*2 /= Temp then
             So_Far := (So_Far*MP) mod Prime;
          end if;
          MP := (MP*MP) mod Prime;
          Temp := Temp/2;
       end loop;
       return So_Far;
       end Exp;
    begin
    Gen.ix := 9_876_543*Exp(46_340, Long_Long_Integer(Initiator), 2_147_483_579) mod 2_147_483_579;
    Gen.iy := 4_789_321*Exp(22_000, Long_Long_Integer(Initiator), 2_147_483_543) mod 2_147_483_543;
    Gen.iz := 1_345_678_901;
    Gen.iu := 1_901_678_345;
    end Reset;

 procedure Reset(Gen : out Generator) is
    -- use the Clock to reset the Generator
    T: Ada.Calendar.Time :=  Ada.Calendar.Clock;
    Value: Long_Float;
    begin
    Gen.ix := Long_Long_Integer(Ada.Calendar.Month(T));
    Gen.iy := Long_Long_Integer(Ada.Calendar.Day(T));
    Value := Long_Float(Ada.Calendar.Seconds(T));
    Gen.iz := Long_Long_Integer(Value) + 1;
    Gen.iu := abs Long_Long_Integer(Long_Float(1.0E6*(Long_Float(Gen.iz-1) - Value))) + 1; -- multiple to cover range of result
    Random(Gen, Value); -- call to randomise value slightly better!
    Random(Gen, Value);                                                     
    end Reset; 

 function Image (Gen   : in Generator) return String_State is
    Temp : constant String := Long_Long_Integer'Image(Gen.ix) &
                              Long_Long_Integer'Image(Gen.iy) &
                              Long_Long_Integer'Image(Gen.iz) &
                              Long_Long_Integer'Image(Gen.iu);
    Temp2: String_State := (others => ' ');
    begin
    Temp2(1..Temp'Last) := Temp;
    return Temp2;
    end Image;
 
 function Value (Coded_State: String_State) return Generator is
    Gen: Generator;
    package My_IO is new Ada.Text_IO.Integer_IO(Long_Long_Integer);
    Last: Integer;
    begin
    My_IO.Get(Coded_State, Gen.ix, Last);
    if Gen.ix < 1 or Gen.ix >= 2_147_483_579 then
       raise Constraint_Error;
    end if;
    My_IO.Get(Coded_State(Last+1..Coded_State'Last), Gen.iy, Last);
    if Gen.iy < 1 or Gen.iy >= 2_147_483_543 then
       raise Constraint_Error;
    end if;
    My_IO.Get(Coded_State(Last+1..Coded_State'Last), Gen.iz, Last);
    if Gen.iz < 1 or Gen.iz >= 2_147_483_423 then
       raise Constraint_Error;
    end if;
    My_IO.Get(Coded_State(Last+1..Coded_State'Last), Gen.iu, Last);
    if Gen.iu < 1 or Gen.iu >= 2_147_483_123 then
       raise Constraint_Error;
    end if;
    return Gen;
    end Value; 

 function Next_Sequence(Gen   : in Generator) return Generator is
    begin
    return ((46_340 * Gen.ix) mod 2_147_483_579, (22_000 * Gen.iy) mod 2_147_483_543, Gen.iz, Gen.iu);
    end Next_Sequence;

end Version5;


